//
// sfive
//
// sfive - spreadspace streaming statistics suite is a generic
// statistic collection tool for streaming server infrastuctures.
// The system collects and stores meta data like number of views
// and throughput from a number of streaming servers and stores
// it in a global data store.
// The data acquisition is designed to be generic and extensible in
// order to support different streaming software.
// sfive also contains tools and applications to filter and visualize
// live and recorded data.
//
//
// Copyright (C) 2014-2018 Christian Pointner <equinox@spreadspace.org>
//                         Markus Grüneis <gimpf@gimpf.org>
//
// This file is part of sfive.
//
// sfive is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 3
// as published by the Free Software Foundation.
//
// sfive is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with sfive. If not, see <http://www.gnu.org/licenses/>.
//

package main

import (
	"flag"
	"log"
	"os"
	"os/signal"
)

var s5l = log.New(os.Stderr, "[s5proxy]\t", log.LstdFlags)

func main() {
	cf := flag.String("config", "/etc/s5proxy/default.json", "path to the config file")
	help := flag.Bool("help", false, "show usage")

	flag.Parse()

	s5l.Printf("Hello, world.\n")
	if *help {
		flag.Usage()
		return
	}

	conf, err := readConfig(*cf)
	if err != nil {
		s5l.Printf("Error reading config file: %v\n", err)
		os.Exit(1)
	}

	var stats *Stats
	if conf.SFive.Sock != "" {
		stats, err = NewStats(conf)
		if err != nil {
			s5l.Printf("Error creating sfive-stats-module: %v\n", err)
			os.Exit(1)
		}
	}

	var proxy *Proxy
	proxy, err = NewProxy(conf, stats)
	if err != nil {
		s5l.Printf("Error creating HTTP-Proxy: %v\n", err)
		os.Exit(1)
	}

	done := make(chan bool)

	if stats != nil {
		go func() {
			defer func() { done <- true }()
			s5l.Println("running sfive-stats-module")
			if err := stats.Run(); err != nil {
				s5l.Printf("ERROR: sfive-stats-module: %v", err)
			}
			s5l.Println("ERROR: sfive-stats-module stopped")
		}()
	}

	go func() {
		defer func() { done <- true }()
		s5l.Println("running HTTP-Proxy")
		if err := proxy.Run(); err != nil {
			s5l.Printf("ERROR: HTTP-Proxy: %v", err)
		}
		s5l.Println("ERROR: HTTP-Proxy stopped")
	}()

	c := make(chan os.Signal, 1)
	signal.Notify(c, os.Interrupt)

	select {
	case <-c:
		s5l.Println("received interrupt, shutdown")
		return
	case <-done:
		return
	}

}
