//
// sfive
//
// sfive - spreadspace streaming statistics suite is a generic
// statistic collection tool for streaming server infrastuctures.
// The system collects and stores meta data like number of views
// and throughput from a number of streaming servers and stores
// it in a global data store.
// The data acquisition is designed to be generic and extensible in
// order to support different streaming software.
// sfive also contains tools and applications to filter and visualize
// live and recorded data.
//
//
// Copyright (C) 2014-2017 Christian Pointner <equinox@spreadspace.org>
//                         Markus Grüneis <gimpf@gimpf.org>
//
// This file is part of sfive.
//
// sfive is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 3
// as published by the Free Software Foundation.
//
// sfive is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with sfive. If not, see <http://www.gnu.org/licenses/>.
//

package sfive

type PipeInterfaceConfig struct {
	ListenPath string `json:"listen" yaml:"listen" toml:"listen"`
}

type PipegramInterfaceConfig struct {
	ListenPath string `json:"listen" yaml:"listen" toml:"listen"`
}

type WebInterfaceConfig struct {
	ListenAddr string `json:"listen" yaml:"listen" toml:"listen"`
}

type InterfacesConfig struct {
	Pipe     PipeInterfaceConfig     `json:"pipe" yaml:"pipe" toml:"pipe"`
	Pipegram PipegramInterfaceConfig `json:"pipegram" yaml:"pipegram" toml:"pipegram"`
	Web      WebInterfaceConfig      `json:"web" yaml:"web" toml:"web"`
}

type TransformConfig struct {
	Anonymize   bool   `json:"anonymize" yaml:"anonymize" toml:"anonymize"`
	AnonKeyfile string `json:"anonymization-key" yaml:"anonymization-key" toml:"anonymization-key"`
	GeoipDB     string `json:"geo-ip-db" yaml:"geo-ip-db" toml:"geo-ip-db"`
}

type AuthConfig struct {
	Token    string `json:"token" yaml:"token" toml:"token"`
	Username string `json:"username" yaml:"username" toml:"username"`
	Password string `json:"password" yaml:"password" toml:"password"`
}

type SFiveForwardConfig struct {
	URL string `json:"url" yaml:"url" toml:"url"`
	AuthConfig
}

type ESForwardConfig struct {
	URL string `json:"url" yaml:"url" toml:"url"`
	AuthConfig
}

type GraphiteForwardConfig struct {
	Host     string `json:"host" yaml:"host" toml:"host"`
	BasePath string `json:"base-path" yaml:"base-path" toml:"base-path"`
}

type PiwikForwardConfig struct {
	URL     string `json:"url" yaml:"url" toml:"url"`
	SiteURL string `json:"site-url" yaml:"site-url" toml:"site-url"`
	SiteID  uint   `json:"site-id" yaml:"site-id" toml:"site-id"`
	AuthConfig
}

type ForwardsConfig struct {
	SFive         SFiveForwardConfig    `json:"sfive" yaml:"sfive" toml:"sfive"`
	Elasticsearch ESForwardConfig       `json:"elasticsearch" yaml:"elasticsearch" toml:"elasticsearch"`
	Graphite      GraphiteForwardConfig `json:"graphite" yaml:"graphite" toml:"graphite"`
	Piwik         PiwikForwardConfig    `json:"piwik" yaml:"piwik" toml:"piwik"`
}

type SrvConfig struct {
	Interfaces InterfacesConfig `json:"interfaces" yaml:"interfaces" toml:"interfaces"`
	Workers    int              `json:"workers" yaml:"workers" toml:"workers"`
	Transform  TransformConfig  `json:"transform" yaml:"transform" toml:"transform"`
	Store      StoreConfig      `json:"store" yaml:"store" toml:"store"`
	Forwards   ForwardsConfig   `json:"forwards" yaml:"forwards" toml:"forwards"`
}
