//
// sfive
//
// sfive - spreadspace streaming statistics suite is a generic
// statistic collection tool for streaming server infrastuctures.
// The system collects and stores meta data like number of views
// and throughput from a number of streaming servers and stores
// it in a global data store.
// The data acquisition is designed to be generic and extensible in
// order to support different streaming software.
// sfive also contains tools and applications to filter and visualize
// live and recorded data.
//
//
// Copyright (C) 2014-2017 Christian Pointner <equinox@spreadspace.org>
//                         Markus Grüneis <gimpf@gimpf.org>
//
// This file is part of sfive.
//
// sfive is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 3
// as published by the Free Software Foundation.
//
// sfive is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with sfive. If not, see <http://www.gnu.org/licenses/>.
//

package sfive

import (
	"fmt"
	"time"

	"github.com/equinox0815/graphite-golang"
)

func (srv *Server) forwardGraphiteRun(forwardHost string, basePath string) {
tryResync:
	for {
		client, err := graphite.NewGraphiteFromAddress(forwardHost)
		if err != nil {
			s5l.Printf("srv|fwd-graphite: connecting failed: %v", err)
			time.Sleep(5 * time.Second)
			continue tryResync
		}

		lastID, err := srv.store.GetLastUpdateID()
		if err != nil {
			s5l.Printf("srv|fwd-graphite: fetching lastupdate failed: %v", err)
			client.Disconnect()
			time.Sleep(5 * time.Second)
			continue tryResync
		}
		s5l.Printf("srv|fwd-graphite: new lastupdate: %d", lastID)

	nextBatch:
		for {
			updates, err := srv.store.GetUpdatesAfter(lastID, 5000)
			if err != nil {
				s5l.Printf("srv|fwd-graphite: reading updates failed: %v", err)
				time.Sleep(500 * time.Millisecond)
				continue nextBatch
			}
			if len(updates) == 0 {
				time.Sleep(1 * time.Second)
				continue nextBatch
			}

			metrics := make([]graphite.Metric, len(updates)*3)
			for i, update := range updates {
				path := basePath + "." + update.Hostname
				path = path + "." + update.Stream.ContentID
				path = path + "." + update.Stream.Format
				path = path + "." + update.Stream.Quality

				metrics[i*3] = graphite.NewMetric(path+".client-count", fmt.Sprintf("%d", update.Data.ClientCount), update.StartTime.Unix())
				metrics[i*3+1] = graphite.NewMetric(path+".bytes-received", fmt.Sprintf("%d", update.Data.BytesReceived), update.StartTime.Unix())
				metrics[i*3+2] = graphite.NewMetric(path+".bytes-sent", fmt.Sprintf("%d", update.Data.BytesSent), update.StartTime.Unix())
			}

			err = client.SendMetrics(metrics)
			if err != nil {
				s5l.Printf("srv|fwd-graphite: sending metrics failed: %v", err)
				time.Sleep(1 * time.Second)
				continue tryResync
			}

			lastID = findMaxID(updates)
			s5l.Printf("srv|fwd-graphite: successfully forwarded %d updates, new lastid: %d", len(updates), lastID)
		}
	}
}

func (srv *Server) RunForwardingGraphite(cfg GraphiteForwardConfig) {
	s5l.Printf("srv|fwd-graphite: forwarding to '%s'", cfg.Host)
	defer s5l.Println("srv|fwd-graphite: forwarder stopped")

	srv.forwardGraphiteRun(cfg.Host, cfg.BasePath)
}
