//
// sfive
//
// sfive - spreadspace streaming statistics suite is a generic
// statistic collection tool for streaming server infrastuctures.
// The system collects and stores meta data like number of views
// and throughput from a number of streaming servers and stores
// it in a global data store.
// The data acquisition is designed to be generic and extensible in
// order to support different streaming software.
// sfive also contains tools and applications to filter and visualize
// live and recorded data.
//
//
// Copyright (C) 2014-2017 Christian Pointner <equinox@spreadspace.org>
//                         Markus Grüneis <gimpf@gimpf.org>
//
// This file is part of sfive.
//
// sfive is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 3
// as published by the Free Software Foundation.
//
// sfive is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with sfive. If not, see <http://www.gnu.org/licenses/>.
//

package sfive

import (
	maxmind "github.com/rainycape/geoip"
)

type GeoIPLookup interface {
	String() string
	Lookup(addr string) (*GeoInfo, error)
}

//
// MaxMind GeoIP2
//

type MaxMindGeoIP2 struct {
	file string
	db   *maxmind.GeoIP
}

func (mm *MaxMindGeoIP2) String() string {
	return "MaxMind GeoIP2 database (" + mm.file + "), last updated: " + mm.db.Updated().String()
}

func (mm *MaxMindGeoIP2) Lookup(addr string) (*GeoInfo, error) {
	rec, err := mm.db.Lookup(addr)
	if err != nil {
		return nil, err
	}

	info := &GeoInfo{}
	if rec.Country != nil {
		info.CountryName = rec.Country.Name.String()
		info.CountryCode2 = rec.Country.Code
	}
	if len(rec.Subdivisions) > 0 && rec.Subdivisions[0] != nil {
		info.RegionName = rec.Subdivisions[0].Name.String()
		info.RegionCode = rec.Subdivisions[0].Code
	}
	if rec.City != nil {
		info.CityName = rec.City.Name.String()
	}
	info.Latitude = rec.Latitude
	info.Longitude = rec.Longitude

	return info, nil
}

func NewMaxMindGeoIP2(dbFile string) (GeoIPLookup, error) {
	var err error
	mm := &MaxMindGeoIP2{file: dbFile}
	mm.db, err = maxmind.Open(dbFile)
	return mm, err
}
